@@ -54,49 +54,51 @@ class Application:
 
         self._api_version = Version(api_version)  # type: Version
 
-        self._app_name = name #type: str
+        self._app_name = name  # type: str
         self._app_display_name = app_display_name if app_display_name else name  # type: str
-        self._version = version #type: str
-        self._build_type = build_type #type: str
-        self._is_debug_mode = is_debug_mode #type: bool
-        self._is_headless = False #type: bool
-        self._use_external_backend = False #type: bool
+        self._version = version  # type: str
+        self._build_type = build_type  # type: str
+        self._is_debug_mode = is_debug_mode  # type: bool
+        self._is_headless = False  # type: bool
+        self._use_external_backend = False  # type: bool
 
-        self._config_lock_filename = "{name}.lock".format(name = self._app_name) # type: str
+        self._just_updated_from_old_version = False  # type: bool
 
-        self._cli_args = None #type: argparse.Namespace
-        self._cli_parser = argparse.ArgumentParser(prog = self._app_name, add_help = False) #type: argparse.ArgumentParser
+        self._config_lock_filename = "{name}.lock".format(name = self._app_name)  # type: str
 
-        self._main_thread = threading.current_thread() #type: threading.Thread
+        self._cli_args = None  # type: argparse.Namespace
+        self._cli_parser = argparse.ArgumentParser(prog = self._app_name, add_help = False)  # type: argparse.ArgumentParser
 
-        self.default_theme = self._app_name  #type: str # Default theme is the application name
-        self._default_language = "en_US" #type: str
+        self._main_thread = threading.current_thread()  # type: threading.Thread
 
-        self.change_log_url = "https://github.com/Ultimaker/Uranium" # Where to find a more detailed description of the recent updates.
+        self.default_theme = self._app_name  # type: str # Default theme is the application name
+        self._default_language = "en_US"  # type: str
 
-        self._preferences_filename = None #type: str
-        self._preferences = None #type: Preferences
+        self.change_log_url = "https://github.com/Ultimaker/Uranium"  # Where to find a more detailed description of the recent updates.
 
-        self._extensions = [] #type: List[Extension]
-        self._required_plugins = [] #type: List[str]
+        self._preferences_filename = None  # type: str
+        self._preferences = None  # type: Preferences
+
+        self._extensions = []  # type: List[Extension]
+        self._required_plugins = []  # type: List[str]
 
         self._package_manager_class = PackageManager  # type: type
         self._package_manager = None  # type: PackageManager
 
-        self._plugin_registry = None #type: PluginRegistry
-        self._container_registry_class = ContainerRegistry #type: type
-        self._container_registry = None #type: ContainerRegistry
-        self._global_container_stack = None #type: ContainerStack
+        self._plugin_registry = None  # type: PluginRegistry
+        self._container_registry_class = ContainerRegistry  # type: type
+        self._container_registry = None  # type: ContainerRegistry
+        self._global_container_stack = None  # type: ContainerStack
 
-        self._controller = None #type: Controller
-        self._backend = None #type: Backend
-        self._output_device_manager = None #type: OutputDeviceManager
-        self._operation_stack = None #type: OperationStack
+        self._controller = None  # type: Controller
+        self._backend = None  # type: Backend
+        self._output_device_manager = None  # type: OutputDeviceManager
+        self._operation_stack = None  # type: OperationStack
 
-        self._visible_messages = [] #type: List[Message]
-        self._message_lock = threading.Lock() #type: threading.Lock
+        self._visible_messages = []  # type: List[Message]
+        self._message_lock = threading.Lock()  # type: threading.Lock
 
-        self._app_install_dir = self.getInstallPrefix() #type: str
+        self._app_install_dir = self.getInstallPrefix()  # type: str
 
     def getAPIVersion(self) -> "Version":
         return self._api_version
@@ -191,9 +193,6 @@ class Application:
         UM.Settings.InstanceContainer.setContainerRegistry(self._container_registry)
         UM.Settings.ContainerStack.setContainerRegistry(self._container_registry)
 
-        # Initialize the package manager to remove and install scheduled packages.
-        self._package_manager = self._package_manager_class(self)
-
         self.showMessageSignal.connect(self.showMessage)
         self.hideMessageSignal.connect(self.hideMessage)
 
@@ -203,6 +202,10 @@ class Application:
     def startPostSplashWindowPhase(self) -> None:
         pass
 
+    # Indicates if we have just updated from an older application version.
+    def hasJustUpdatedFromOldVersion(self) -> bool:
+        return self._just_updated_from_old_version
+
     ##  Run the main event loop.
     #   This method should be re-implemented by subclasses to start the main event loop.
     #   \exception NotImplementedError
@@ -398,9 +401,10 @@ class Application:
     @staticmethod
     def getInstallPrefix() -> str:
         if "python" in os.path.basename(sys.executable):
-            return os.path.abspath(os.path.join(os.path.dirname(sys.argv[0]), ".."))
+            executable = sys.argv[0]
         else:
-            return os.path.abspath(os.path.join(os.path.dirname(sys.executable), ".."))
+            executable = sys.executable
+        return os.path.abspath(os.path.join(os.path.dirname(os.path.realpath(executable)), ".."))
 
     __instance = None   # type: Application
 
