@@ -9,7 +9,6 @@ from UM.Logger import Logger
 from UM.Math import NumPyUtil
 from UM.Math import ShapelyUtil
 
-
 ##  A class representing an immutable arbitrary 2-dimensional polygon.
 class Polygon:
     ##  Return vertices from an approximate circle.
@@ -57,7 +56,7 @@ class Polygon:
         return "[" + ", ".join(coordinates) + "]"
 
     def isValid(self) -> bool:
-        return bool(self._points is not None and len(self._points))
+        return bool(self._points is not None and len(self._points) >= 3)
 
     def getPoints(self):
         return self._points
@@ -142,6 +141,28 @@ class Polygon:
 
         return Polygon(points = [list(p) for p in polygon_intersection.exterior.coords[:4]])
 
+    #  Computes the convex hull of the union of the convex hulls of this and another polygon.
+    #
+    #   \param other The other polygon to combine convex hulls with.
+    #   \return The convex hull of the union of the two polygons' convex hulls.
+    def unionConvexHulls(self, other: "Polygon") -> "Polygon":
+        my_hull = self.getConvexHull()
+        other_hull = other.getConvexHull()
+
+        if not my_hull.isValid():
+            return other_hull
+        if not other_hull.isValid():
+            return my_hull
+
+        my_polygon = ShapelyUtil.polygon2ShapelyPolygon(my_hull)
+        other_polygon = ShapelyUtil.polygon2ShapelyPolygon(other_hull)
+
+        polygon_union = my_polygon.union(other_polygon).convex_hull
+        if polygon_union.area == 0:
+            return Polygon()
+
+        return Polygon(points = [list(p) for p in polygon_union.exterior.coords[:-1]])
+
     ##  Check to see whether this polygon intersects with another polygon.
     #
     #   \param other \type{Polygon} The polygon to check for intersection.
@@ -155,6 +176,9 @@ class Polygon:
         polygon_me = ShapelyUtil.polygon2ShapelyPolygon(self)
         polygon_other = ShapelyUtil.polygon2ShapelyPolygon(other)
 
+        if not (polygon_me.is_valid and polygon_other.is_valid):
+            return None
+
         polygon_intersection = polygon_me.intersection(polygon_other)
         ret_size = None
         if polygon_intersection:
