@@ -146,7 +146,7 @@ class PluginRegistry(QObject):
         except:
             # Since we're writing to file (and waiting for a lock), there are a few things that can go wrong.
             # There is no need to crash the application for this, but it is a failure that we want to log.
-            Logger.log("e", "Unable to save the plugin data.")
+            Logger.logException("e", "Unable to save the plugin data.")
 
     # TODO:
     # - [ ] Improve how metadata is stored. It should not be in the 'plugin' prop
@@ -165,29 +165,6 @@ class PluginRegistry(QObject):
     # PUBLIC METHODS
     #===============================================================================
 
-    #   If used, this can add available plugins (from a remote server) to the
-    #   registry. Cura uses this method to add 3rd-party plugins.
-    def addExternalPlugins(self, plugin_list: List[Dict[str, Any]]) -> None:
-        for plugin in plugin_list:
-            # Add the plugin id to the all plugins list if not already there:
-            if plugin["id"] not in self._all_plugins:
-                self._all_plugins.append(plugin["id"])
-
-                # Does this look redundant?
-                # It is. Should be simplfied in the future but changing it right
-                # now may break other functionality.
-                if plugin["id"] not in self._plugins_available:
-                    self._plugins_available.append(plugin["id"])
-            self._metadata[plugin["id"]] = {
-                "id": plugin["id"],
-                "plugin": plugin,
-                "update_url": plugin["file_location"]
-            }
-
-            # Keep a note of plugins which are not Ultimaker plugins:
-            if plugin["id"] not in self._plugins_external:
-                self._plugins_external.append(plugin["id"])
-
     #   Add a plugin location to the list of locations to search:
     def addPluginLocation(self, location: str) -> None:
         #TODO: Add error checking!
@@ -262,7 +239,7 @@ class PluginRegistry(QObject):
         for plugin_id in self._plugins_to_install:
             if plugin_id not in plugins:
                 plugins.append(plugin_id)
-        return plugins
+        return sorted(plugins)
 
     #   Get the metadata for a certain plugin:
     #   NOTE: InvalidMetaDataError is raised when no metadata can be found or
@@ -314,19 +291,11 @@ class PluginRegistry(QObject):
 
     #   Check by ID if a plugin is active (enabled):
     def isActivePlugin(self, plugin_id: str) -> bool:
-        if plugin_id not in self._disabled_plugins and plugin_id not in self._outdated_plugins:
+        if plugin_id not in self._disabled_plugins and plugin_id not in self._outdated_plugins and plugin_id in self._all_plugins:
             return True
 
         return False
 
-    #   Check by ID if a plugin is available:
-    def isAvailablePlugin(self, plugin_id: str) -> bool:
-        return plugin_id in self._plugins_available
-
-    #   Check by ID if a plugin is installed:
-    def isInstalledPlugin(self, plugin_id: str) -> bool:
-        return plugin_id in self._plugins_installed
-
     def isBundledPlugin(self, plugin_id: str) -> bool:
         if plugin_id in self._bundled_plugin_cache:
             return self._bundled_plugin_cache[plugin_id]
@@ -353,7 +322,7 @@ class PluginRegistry(QObject):
     #   \param meta_data \type{dict} The meta data that needs to be matched.
     #   \sa loadPlugin
     #   NOTE: This is the method which kicks everything off at app launch.
-    def loadPlugins(self, metadata: Optional[dict] = None) -> None:
+    def loadPlugins(self, metadata: Optional[Dict[str, Any]] = None) -> None:
         # Get a list of all installed plugins:
         plugin_ids = self._findInstalledPlugins()
         for plugin_id in plugin_ids:
@@ -444,6 +413,9 @@ class PluginRegistry(QObject):
         result = {"status": "error", "message": "", "id": plugin_id}
         success_message = i18n_catalog.i18nc("@info:status", "The plugin has been removed.\nPlease restart {0} to finish uninstall.", self._application.getApplicationName())
 
+        if plugin_id not in self._plugins_installed:
+            return result
+
         in_to_install = plugin_id in self._plugins_to_install
         if in_to_install:
             del self._plugins_to_install[plugin_id]
@@ -517,7 +489,10 @@ class PluginRegistry(QObject):
                         break
         except zipfile.BadZipFile:
             Logger.logException("e", "Failed to load plug-in file. The zip archive seems to be corrupt.")
-            return None #Signals that loading this failed.
+            return None  # Signals that loading this failed.
+        except FileNotFoundError:
+            Logger.logException("e", "Failed to load plug-in file as we were unable to find it.")
+            return None  # Signals that loading this failed.
         return plugin_id
 
     #   Returns a list of all possible plugin ids in the plugin locations:
@@ -594,6 +569,41 @@ class PluginRegistry(QObject):
 
         return None
 
+    #   Load the plugin data from the stream and in-place update the metadata.
+    def _parsePluginInfo(self, plugin_id, file_data, meta_data):
+        try:
+            meta_data["plugin"] = json.loads(file_data)
+        except json.decoder.JSONDecodeError:
+            Logger.logException("e", "Failed to parse plugin.json for plugin %s", plugin_id)
+            raise InvalidMetaDataError(plugin_id)
+
+        # Check if metadata is valid;
+        if "version" not in meta_data["plugin"]:
+            Logger.log("e", "Version must be set!")
+            raise InvalidMetaDataError(plugin_id)
+
+        # Check if the plugin states what API version it needs.
+        if "api" not in meta_data["plugin"] and "supported_sdk_versions" not in meta_data["plugin"]:
+            Logger.log("e", "The API or the supported_sdk_versions must be set!")
+            raise InvalidMetaDataError(plugin_id)
+        else:
+            # Store the api_version as a Version object.
+            all_supported_sdk_versions = []  # type: List[Version]
+            if "supported_sdk_versions" in meta_data["plugin"]:
+                all_supported_sdk_versions += [Version(supported_version) for supported_version in
+                                               meta_data["plugin"]["supported_sdk_versions"]]
+            if "api" in meta_data["plugin"]:
+                all_supported_sdk_versions += [Version(meta_data["plugin"]["api"])]
+            meta_data["plugin"]["supported_sdk_versions"] = all_supported_sdk_versions
+
+        if "i18n-catalog" in meta_data["plugin"]:
+            # A catalog was set, try to translate a few strings
+            i18n_catalog = i18nCatalog(meta_data["plugin"]["i18n-catalog"])
+            if "name" in meta_data["plugin"]:
+                meta_data["plugin"]["name"] = i18n_catalog.i18n(meta_data["plugin"]["name"])
+            if "description" in meta_data["plugin"]:
+                meta_data["plugin"]["description"] = i18n_catalog.i18n(meta_data["plugin"]["description"])
+
     ##  private:
     #   Populate the list of metadata
     #   \param plugin_id \type{string}
@@ -620,39 +630,8 @@ class PluginRegistry(QObject):
 
             metadata_file = os.path.join(location, "plugin.json")
             try:
-                with open(metadata_file, "r", encoding = "utf-8") as f:
-                    try:
-                        meta_data["plugin"] = json.loads(f.read())
-                    except json.decoder.JSONDecodeError:
-                        Logger.logException("e", "Failed to parse plugin.json for plugin %s", plugin_id)
-                        raise InvalidMetaDataError(plugin_id)
-
-                    # Check if metadata is valid;
-                    if "version" not in meta_data["plugin"]:
-                        Logger.log("e", "Version must be set!")
-                        raise InvalidMetaDataError(plugin_id)
-
-                    # Check if the plugin states what API version it needs.
-                    if "api" not in meta_data["plugin"] and "supported_sdk_versions" not in meta_data["plugin"]:
-                        Logger.log("e", "The API or the supported_sdk_versions must be set!")
-                        raise InvalidMetaDataError(plugin_id)
-                    else:
-                        # Store the api_version as a Version object.
-                        all_supported_sdk_versions = []  # type: List[Version]
-                        if "supported_sdk_versions" in meta_data["plugin"]:
-                            all_supported_sdk_versions += [Version(supported_version) for supported_version in meta_data["plugin"]["supported_sdk_versions"]]
-                        if "api" in meta_data["plugin"]:
-                            all_supported_sdk_versions += [Version(meta_data["plugin"]["api"])]
-                        meta_data["plugin"]["supported_sdk_versions"] = all_supported_sdk_versions
-
-                    if "i18n-catalog" in meta_data["plugin"]:
-                        # A catalog was set, try to translate a few strings
-                        i18n_catalog = i18nCatalog(meta_data["plugin"]["i18n-catalog"])
-                        if "name" in meta_data["plugin"]:
-                             meta_data["plugin"]["name"] = i18n_catalog.i18n(meta_data["plugin"]["name"])
-                        if "description" in meta_data["plugin"]:
-                            meta_data["plugin"]["description"] = i18n_catalog.i18n(meta_data["plugin"]["description"])
-
+                with open(metadata_file, "r", encoding = "utf-8") as file_stream:
+                    self._parsePluginInfo(plugin_id, file_stream.read(), meta_data)
             except FileNotFoundError:
                 Logger.logException("e", "Unable to find the required plugin.json file for plugin %s", plugin_id)
                 raise InvalidMetaDataError(plugin_id)
