@@ -1,13 +1,14 @@
 # Copyright (c) 2015 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
 
-from PyQt5.QtCore import QObject, QCoreApplication, pyqtSlot, QUrl, pyqtSignal, pyqtProperty
+from PyQt5.QtCore import QObject, pyqtSlot, pyqtSignal, pyqtProperty
 
 from UM.Application import Application
 from UM.Scene.Selection import Selection
 from UM.Operations.RemoveSceneNodeOperation import RemoveSceneNodeOperation
 from UM.Operations.GroupedOperation import GroupedOperation
 
+
 class ControllerProxy(QObject):
     def __init__(self, parent = None):
         super().__init__(parent)
@@ -61,22 +62,10 @@ class ControllerProxy(QObject):
         op.push()
         Selection.clear()
 
-    @pyqtSlot()
-    def enableModelRendering(self):
-        self._controller.enableModelRendering()
-
-    @pyqtSlot()
-    def disableModelRendering(self):
-        self._controller.disableModelRendering()
-
     @pyqtSlot(str, int)
     def rotateView(self,coordinate, angle):
         self._controller.rotateView(coordinate, angle)
 
-    @pyqtSlot()
-    def homeView(self, angle):
-        self._controller.homeView()
-
     contextMenuRequested = pyqtSignal("quint64", arguments=["objectId"])
 
     def _onContextMenuRequested(self, x, y):
