@@ -1,5 +1,5 @@
-# Copyright (c) 2015 Ultimaker B.V.
-# Uranium is released under the terms of the AGPLv3 or higher.
+# Copyright (c) 2017 Ultimaker B.V.
+# Uranium is released under the terms of the LGPLv3 or higher.
 
 from PyQt5.QtCore import QObject, pyqtSignal, pyqtProperty, pyqtSlot
 
@@ -21,8 +21,19 @@ class SelectionProxy(QObject):
     def selectionCount(self):
         return Selection.getCount()
 
+    @pyqtProperty("QVariantList", notify = selectionChanged)
+    def selectionNames(self):
+        return [node.getName() for node in Selection.getAllSelectedObjects()]
+
     def _onSelectionChanged(self):
         self.selectionChanged.emit()
 
+    @pyqtProperty(bool, notify=selectionChanged)
+    def isGroupSelected(self):
+        for node in Selection.getAllSelectedObjects():
+            if node.callDecoration("isGroup"):
+                return True
+        return False
+
 def createSelectionProxy(engine, script_engine):
     return SelectionProxy()
