@@ -21,10 +21,6 @@ from UM.Platform import Platform
 
 i18n_catalog = i18nCatalog("uranium")
 
-MYPY = False
-if MYPY:
-    from UM.Scene.SceneNode import SceneNode
-
 
 ##  Container object for the scene graph
 #
@@ -32,10 +28,10 @@ if MYPY:
 @signalemitter
 class Scene:
     def __init__(self) -> None:
-        super().__init__()  # Call super to make multiple inheritance work.
+        super().__init__()
 
         from UM.Scene.SceneNode import SceneNode
-        self._root = SceneNode(name= "Root")
+        self._root = SceneNode(name = "Root")
         self._root.setCalculateBoundingBox(False)
         self._connectSignalsRoot()
         self._active_camera = None  # type: Optional[Camera]
@@ -66,22 +62,6 @@ class Scene:
             else:
                 self._connectSignalsRoot()
 
-    ##  Acquire the global scene lock.
-    #
-    #   This will prevent any read or write actions on the scene from other threads,
-    #   assuming those threads also properly acquire the lock. Most notably, this
-    #   prevents the rendering thread from rendering the scene while it is changing.
-    #   Deprecated, use getSceneLock() instead.
-    @deprecated("Please use the getSceneLock instead", "3.3")
-    def acquireLock(self) -> None:
-        self._lock.acquire()
-
-    ##  Release the global scene lock.
-    #   Deprecated, use getSceneLock() instead.
-    @deprecated("Please use the getSceneLock instead", "3.3")
-    def releaseLock(self) -> None:
-        self._lock.release()
-
     ##  Gets the global scene lock.
     #
     #   Use this lock to prevent any read or write actions on the scene from other threads,
