@@ -46,9 +46,6 @@ class Selection:
     def getBoundingBox(cls) -> AxisAlignedBox:
         bounding_box = None  # don't start with an empty bounding box, because that includes (0,0,0)
         for node in cls.__selection:
-            if not isinstance(node, SceneNode):
-                continue
-
             if not bounding_box:
                 bounding_box = node.getBoundingBox()
             else:
@@ -89,9 +86,6 @@ class Selection:
 
     @classmethod
     def getSelectionCenter(cls) -> Vector:
-        if not cls.__selection:
-            cls.__selection_center = Vector.Null
-
         return cls.__selection_center
 
     ##  Apply an operation to the entire selection
