@@ -1,5 +1,5 @@
 # Copyright (c) 2015 Ultimaker B.V.
-# Uranium is released under the terms of the AGPLv3 or higher.
+# Uranium is released under the terms of the LGPLv3 or higher.
 
 from . import SceneNode
 
@@ -55,6 +55,8 @@ class ToolHandle(SceneNode.SceneNode):
         self._active_axis = None
         self._auto_scale = True
 
+        self._enabled = False
+
         self.setCalculateBoundingBox(False)
 
         Selection.selectionCenterChanged.connect(self._onSelectionCenterChanged)
@@ -81,9 +83,6 @@ class ToolHandle(SceneNode.SceneNode):
         self._selection_mesh = mesh
         self.meshDataChanged.emit(self)
 
-    def getMaterial(self):
-        return self._shader
-
     def render(self, renderer):
         if not self._shader:
             self._shader = OpenGL.getInstance().createShaderProgram(Resources.getPath(Resources.Shaders, "toolhandle.shader"))
@@ -106,12 +105,15 @@ class ToolHandle(SceneNode.SceneNode):
             return
 
         if axis:
-            self._shader.setUniformValue("u_activeColor", self._axis_color_map[axis])
+            self._shader.setUniformValue("u_activeColor", self._axis_color_map.get(axis, Color()))
         else:
             self._shader.setUniformValue("u_activeColor", self._disabled_axis_color)
         self._active_axis = axis
         self._scene.sceneChanged.emit(self)
 
+    def getActiveAxis(self):
+        return self._active_axis
+
     def isAxis(self, value):
         return value in self._axis_color_map
 
@@ -120,7 +122,13 @@ class ToolHandle(SceneNode.SceneNode):
         pass
 
     def _onSelectionCenterChanged(self):
-        self.setPosition(Selection.getSelectionCenter())
+        if self._enabled:
+            self.setPosition(Selection.getSelectionCenter())
+
+    def setEnabled(self, enable: bool):
+        super().setEnabled(enable)
+        # Force an update
+        self._onSelectionCenterChanged()
 
     def _onEngineCreated(self):
         theme = Application.getInstance().getTheme()
