@@ -1,4 +1,4 @@
-# Copyright (c) 2018 Ultimaker B.V.
+# Copyright (c) 2019 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
 
 import configparser
@@ -471,7 +471,7 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
 
     ##  Get a container by index.
     #
-    #   \param index \type{int} The index of the container to get.
+    #   \param index The index of the container to get.
     #
     #   \return The container at the specified index.
     #
@@ -677,7 +677,7 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
                 if validator_type:
                     validator = validator_type(key)
                     validation_state = validator(self)
-            if validation_state in (ValidatorState.Exception, ValidatorState.MaximumError, ValidatorState.MinimumError):
+            if validation_state in (ValidatorState.Exception, ValidatorState.MaximumError, ValidatorState.MinimumError, ValidatorState.Invalid):
                 return True
         return False
 
@@ -696,7 +696,7 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
                 if validator_type:
                     validator = validator_type(key)
                     validation_state = validator(self)
-            if validation_state in (ValidatorState.Exception, ValidatorState.MaximumError, ValidatorState.MinimumError):
+            if validation_state in (ValidatorState.Exception, ValidatorState.MaximumError, ValidatorState.MinimumError, ValidatorState.Invalid):
                 error_keys.append(key)
         return error_keys
 
