@@ -2,22 +2,20 @@
 # Uranium is released under the terms of the LGPLv3 or higher.
 
 from enum import Enum
-from typing import Any, Optional
+from typing import Optional
 
 from UM.Settings.Interfaces import ContainerInterface
 from UM.Settings.PropertyEvaluationContext import PropertyEvaluationContext
 from UM.Logger import Logger
 
-MYPY = False
-if MYPY:
-    from UM.Settings.SettingInstance import SettingInstance
-from typing import Optional
 from . import SettingFunction
 
+
 class ValidatorState(Enum):
     Exception = "Exception"
     Unknown = "Unknown"
     Valid = "Valid"
+    Invalid = "Invalid"
     MinimumError = "MinimumError"
     MinimumWarning = "MinimumWarning"
     MaximumError = "MaximumError"
@@ -47,6 +45,7 @@ class Validator(SettingFunction.SettingFunction):
 
         state = ValidatorState.Unknown
         try:
+            allow_empty = value_provider.getProperty(self._key, "allow_empty")  # For string only
             minimum = value_provider.getProperty(self._key, "minimum_value")
             maximum = value_provider.getProperty(self._key, "maximum_value")
             minimum_warning = value_provider.getProperty(self._key, "minimum_value_warning")
@@ -62,7 +61,11 @@ class Validator(SettingFunction.SettingFunction):
             if value is None or value != value:
                 raise ValueError("Cannot validate None, NaN or similar values in setting {0}, actual value: {1}".format(self._key, value))
 
-            if minimum is not None and value < minimum:
+            # "allow_empty is not None" is not necessary here because of "allow_empty is False", but it states
+            # explicitly that we should not do this check when "allow_empty is None".
+            if allow_empty is not None and allow_empty is False and str(value) == "":
+                state = ValidatorState.Invalid
+            elif minimum is not None and value < minimum:
                 state = ValidatorState.MinimumError
             elif maximum is not None and value > maximum:
                 state = ValidatorState.MaximumError
