@@ -166,10 +166,14 @@ class ScaleTool(Tool):
                             scale_change = scale_change.set(x=scale_factor, y=scale_factor, z=scale_factor)
 
                         # Scale around the saved centers of all selected nodes
-                        op = GroupedOperation()
-                        for node, position in self._saved_node_positions:
-                            op.addOperation(ScaleOperation(node, scale_change, relative_scale = True, scale_around_point = position))
-                        op.push()
+                        if len(self._saved_node_positions) > 1:
+                            op = GroupedOperation()
+                            for node, position in self._saved_node_positions:
+                                op.addOperation(ScaleOperation(node, scale_change, relative_scale = True, scale_around_point = position))
+                            op.push()
+                        else:
+                            for node, position in self._saved_node_positions:
+                                ScaleOperation(node, scale_change, relative_scale = True, scale_around_point = position).push()
                         self._drag_length = (self._saved_handle_position - drag_position).length()
                 else:
                     self.operationStarted.emit(self)
@@ -290,12 +294,7 @@ class ScaleTool(Tool):
                 else:
                     scale_vector = Vector(scale_factor, scale_factor, scale_factor)
 
-                op = GroupedOperation()
-                for node in self._getSelectedObjectsWithoutSelectedAncestors():
-                    op.addOperation(
-                        ScaleOperation(node, scale_vector, scale_around_point=node.getWorldPosition()))
-                op.push()
-
+                self._scaleSelectedNodes(scale_vector)
     ##  Set the height of the selected object(s) by scaling the first selected object to a certain height
     #
     #   \param height type(float) height in mm
@@ -311,11 +310,7 @@ class ScaleTool(Tool):
                 else:
                     scale_vector = Vector(scale_factor, scale_factor, scale_factor)
 
-                op = GroupedOperation()
-                for node in self._getSelectedObjectsWithoutSelectedAncestors():
-                    op.addOperation(
-                        ScaleOperation(node, scale_vector, scale_around_point=node.getWorldPosition()))
-                op.push()
+                self._scaleSelectedNodes(scale_vector)
 
     ##  Set the depth of the selected object(s) by scaling the first selected object to a certain depth
     #
@@ -332,11 +327,7 @@ class ScaleTool(Tool):
                 else:
                     scale_vector = Vector(scale_factor, scale_factor, scale_factor)
 
-                op = GroupedOperation()
-                for node in self._getSelectedObjectsWithoutSelectedAncestors():
-                    op.addOperation(
-                        ScaleOperation(node, scale_vector, scale_around_point=node.getWorldPosition()))
-                op.push()
+                self._scaleSelectedNodes(scale_vector)
 
     ##  Set the x-scale of the selected object(s) by scaling the first selected object to a certain factor
     #
@@ -352,11 +343,7 @@ class ScaleTool(Tool):
                 else:
                     scale_vector = Vector(scale_factor, scale_factor, scale_factor)
 
-                op = GroupedOperation()
-                for node in self._getSelectedObjectsWithoutSelectedAncestors():
-                    op.addOperation(
-                        ScaleOperation(node, scale_vector, scale_around_point=node.getWorldPosition()))
-                op.push()
+                self._scaleSelectedNodes(scale_vector)
 
     ##  Set the y-scale of the selected object(s) by scaling the first selected object to a certain factor
     #
@@ -372,11 +359,7 @@ class ScaleTool(Tool):
                 else:
                     scale_vector = Vector(scale_factor, scale_factor, scale_factor)
 
-                op = GroupedOperation()
-                for node in self._getSelectedObjectsWithoutSelectedAncestors():
-                    op.addOperation(
-                        ScaleOperation(node, scale_vector, scale_around_point=node.getWorldPosition()))
-                op.push()
+                self._scaleSelectedNodes(scale_vector)
 
     ##  Set the z-scale of the selected object(s) by scaling the first selected object to a certain factor
     #
@@ -392,11 +375,18 @@ class ScaleTool(Tool):
                 else:
                     scale_vector = Vector(scale_factor, scale_factor, scale_factor)
 
-                op = GroupedOperation()
-                for node in self._getSelectedObjectsWithoutSelectedAncestors():
-                    op.addOperation(
-                        ScaleOperation(node, scale_vector, scale_around_point=node.getWorldPosition()))
-                op.push()
+                self._scaleSelectedNodes(scale_vector)
+
+    def _scaleSelectedNodes(self, scale_vector: Vector) -> None:
+        selected_nodes = self._getSelectedObjectsWithoutSelectedAncestors()
+        if len(selected_nodes) > 1:
+            op = GroupedOperation()
+            for node in selected_nodes:
+                op.addOperation(ScaleOperation(node, scale_vector, scale_around_point=node.getWorldPosition()))
+            op.push()
+        else:
+            for node in selected_nodes:
+                ScaleOperation(node, scale_vector, scale_around_point=node.getWorldPosition()).push()
 
     ##  Convenience function that gives the scale of an object in the coordinate space of the world.
     #   The function might return wrong value if the grouped models are rotated
